;################################################
;                                               #
;                       MENU                    #
;                                               #
; Displays a menu of text options with an arrow #
; cursor.  Reponds to UP,DOWN,2ND,ALPHA keys.   #
;                                               #
; Normal Mode: Menu is displayed until a        #
; selection is made.  The latest keypress and   #
; cursor position is returned                   #
;                                               #
; Live Mode: Cursor position is constantly      #
; returned until a selection is made.  Program  #
; execution is not halted.                      #
;                                               #
; IN: HL= (menu strings)                        #
;     DE= vertical position of text:left margin #
;      A= number of choices                     #
;                                               #
; OUT: A= Latest keypress                       #
;      C= Latest cursor position                #
;                                               #
;------------------------------------------------
;                                               #
;                   LIVE MENU                   #
;                                               #
; Live mode menu is designed to operate within  #
; a running loop.  HL and DE need to be setup   #
; BEFORE the loop begins.  C and A are          #
; constantly updated with the latest keypresses #
; as the loop runs.  LIVE MENU does not display #
; text                                          #
;                                               #
; IN:  A= number of choices                     #
;                                               #
; UPDATED CONSTANTLY:                           #
;      A= latest keypress                       #
;      C= current cursor position               #
;                                               #
; DECLARED PRIOR TO LOOP:                       #
;     DE= vertical position : left margin       #
;      C= initial cursor position (1=default)   #
;                                               #
;################################################
MENU:
 ld (menu_limit),a      ; Set cursor limit 
 push de                ; Save positions
 call PARAGRAPH         ; display the choices
 pop de                 ; Restore positions
 ld a,e                 ; Set left margin for the arrow
 sub 6                  ;
 ld e,a                 ;
 ld c,1                 ; C = cursor's initial position (1 typical)
menu_loop0:
 push de
 ld hl,arrow
 call DMX2
 EI
 halt
 halt
 halt
 DI
 call CR_GRBCopy
 ld hl,blit             ; Blit out the arrow is case the user is about...
 call DMX2              ; ...to use UP or DOWN
 call GET_KEY           ; Scan for UP/DOWN/2ND/ALPHA
 pop de
 cp G_2ND               ; Ok
 ret z
 cp G_ALPHA             ; Cancel
 ret z
 cp G_UP
 jr z,menu_up
 cp G_DOWN
 jr nz,menu_loop0       ; Keep looping until UP/DOWN/2ND/ALPHA is used
menu_down:
 ld a,c                 ; C = current cursor position
menu_limit = $+1
 cp $00                 ; <-- cursor limit (polymorphic)
 jr z,menu_loop0        ; Stop if already at the limit
 ld a,d                 ; Adjust cursor vertically
 add a,6                ;
 ld d,a                 ;
 inc c
 jr menu_loop0
menu_up:
 ld a,c                 ; C = current cursor position
 cp 1                   ; "Is cursor at uppermost limit?"
 jr z,menu_loop0        ; Stop if already at the limit
 ld a,d                 ; Adjust cursor vertically
 sub 6                  ;
 ld d,a                 ;
 dec c
 jr menu_loop0

MENU_LIVE:
 ld (menu_live_limit),a ; Set cursor limit
 ld hl,arrow            ; Display cursor at appropriate position
 call DMX2              ;
 call CR_GRBCopy        ;
 ld hl,blit             ; Blank out cursor so it can be moved if needed
 call DMX2              ;
 call GET_KEY
 cp G_UP                ; Adjust cursor and C
 jr z,menu_live_up
 cp G_DOWN              ; Adjust cursor and C
 ret nz
menu_live_down:
 ld a,c                 ; C = current cursor position
menu_live_limit = $+1
 cp $00                 ; "Is cursor at lowest point?"
 ret z                  ; If so, don't adjust
 ld a,d                 ; Modify cursor and blit positions
 add a,6                ;
 ld d,a                 ;
 xor a                  ; Set a=0 so it's not mistaken for a keystroke
 inc c                  ;
 ret
menu_live_up:
 ld a,c                 ; C = current cursor position
 cp 1                   ; "Is cursor at uppermost point?"
 ret z                  ; If so, don't adjust
 ld a,d                 ; Modify cursor and blit positions
 sub 6                  ;
 ld d,a                 ;
 xor a                  ; Set a=0 so it's not mistaken for a keystroke
 dec c                  ;
 ret

menu_null:
.db 0,$FF

arrow:
.db $05,0
str_close_blit:
.db ">"
blit:
blit2 = $ + 2
.db "   ",0
